"""Ariston module"""
import asyncio
import logging
from typing import Any, Optional

from .ariston_api import AristonAPI, ConnectionException
from .const import (
    ARISTON_API_URL,
    DeviceAttribute,
    SystemType,
    VelisDeviceAttribute,
    WheType,
)
from .bsb_device import AristonBsbDevice
from .lux_device import AristonLuxDevice
from .lux2_device import AristonLux2Device
from .evo_one_device import AristonEvoOneDevice
from .evo_device import AristonEvoDevice
from .galevo_device import AristonGalevoDevice
from .lydos_hybrid_device import AristonLydosHybridDevice
from .nuos_split_device import AristonNuosSplitDevice
from .base_device import AristonBaseDevice

_LOGGER = logging.getLogger(__name__)

_MAP_WHE_TYPES_TO_CLASS = {
    WheType.Evo.value: AristonEvoOneDevice,
    WheType.LydosHybrid.value: AristonLydosHybridDevice,
    WheType.Lydos.value: AristonEvoDevice,
    WheType.NuosSplit.value: AristonNuosSplitDevice,
    WheType.Andris2.value: AristonEvoDevice,
    WheType.Evo2.value: AristonEvoDevice,
    WheType.Lux2.value: AristonLux2Device,
    WheType.Lux.value: AristonLuxDevice,
}

class Ariston:
    """Ariston class"""

    def __init__(self) -> None:
        self.api = None
        self.cloud_devices: list[dict[str, Any]] = []

    async def async_connect(
        self, username: str, password: str, api_url: str = ARISTON_API_URL
    ) -> bool:
        """Connect to the ariston cloud"""
        self.api = AristonAPI(username, password, api_url)
        return await self.api.async_connect()

    async def async_discover(self) -> Optional[list[dict[str, Any]]]:
        """Retreive ariston devices from the cloud"""
        if self.api is None:
            _LOGGER.exception("Call async_connect first")
            return None
        cloud_devices = await _async_discover(self.api)
        self.cloud_devices = cloud_devices
        return cloud_devices

    async def async_hello(
        self, gateway: str, is_metric: bool = True, language_tag: str = "en-US"
    ) -> Optional[AristonBaseDevice]:
        """Get ariston device"""
        if self.api is None:
            _LOGGER.exception("Call async_connect() first")
            return None

        if len(self.cloud_devices) == 0:
            await self.async_discover()

        return _get_device(
            self.cloud_devices, self.api, gateway, is_metric, language_tag
        )


def _get_device(
    cloud_devices: list[dict[str, Any]],
    api: AristonAPI,
    gateway: str,
    is_metric: bool = True,
    language_tag: str = "en-US",
) -> Optional[AristonBaseDevice]:
    """Get ariston device"""
    device = next(
        (dev for dev in cloud_devices if dev.get(DeviceAttribute.GW) == gateway),
        None,
    )
    if device is None:
        _LOGGER.exception("No device %s found.", gateway)
        return None

    system_type = device.get(DeviceAttribute.SYS)

    match system_type:
        case SystemType.GALEVO.value:
            return AristonGalevoDevice(
                api,
                device,
                is_metric,
                language_tag,
            )
        case SystemType.VELIS.value:
            whe_type = device.get(VelisDeviceAttribute.WHE_TYPE, None)
            device_class = _MAP_WHE_TYPES_TO_CLASS.get(whe_type, None)
            if device_class is None:
                _LOGGER.exception("Unsupported whe type %s", whe_type)
                return None
            return device_class(api, device)

        case SystemType.BSB.value:
            return AristonBsbDevice(api, device)
        case _:
            _LOGGER.exception("Unsupported system type %s", system_type)
            return None


def _connect(username: str, password: str) -> AristonAPI:
    """Connect to ariston api"""
    api = AristonAPI(username, password)
    api.connect()
    return api


def _discover(api: AristonAPI) -> list[dict[str, Any]]:
    """Retreive ariston devices from the cloud"""
    cloud_devices: list[dict[str, Any]] = []
    cloud_devices.extend(api.get_detailed_devices())
    cloud_devices.extend(api.get_detailed_velis_devices())

    return cloud_devices


def discover(username: str, password: str) -> list[dict[str, Any]]:
    """Retreive ariston devices from the cloud"""
    api = _connect(username, password)
    return _discover(api)


def hello(
    username: str,
    password: str,
    gateway: str,
    is_metric: bool = True,
    language_tag: str = "en-US",
) -> Optional[AristonBaseDevice]:
    """Get ariston device"""
    api = _connect(username, password)
    cloud_devices = _discover(api)
    return _get_device(cloud_devices, api, gateway, is_metric, language_tag)


async def _async_connect(username: str, password: str) -> AristonAPI:
    """Async connect to ariston api"""
    api = AristonAPI(username, password)
    if not await api.async_connect():
        raise ConnectionException
    return api


async def _async_discover(api: AristonAPI) -> list[dict[str, Any]]:
    """Async retreive ariston devices from the cloud"""
    cloud_devices: list[dict[str, Any]] = []
    cloud_devices_tuple: tuple[
        list[dict[str, Any]], list[dict[str, Any]]
    ] = await asyncio.gather(
        api.async_get_detailed_devices(), api.async_get_detailed_velis_devices()
    )

    for devices in cloud_devices_tuple:
        cloud_devices.extend(devices)

    return cloud_devices


async def async_discover(username: str, password: str) -> list[dict[str, Any]]:
    """Retreive ariston devices from the cloud"""
    api = await _async_connect(username, password)
    return await _async_discover(api)


async def async_hello(
    username: str,
    password: str,
    gateway: str,
    is_metric: bool = True,
    language_tag: str = "en-US",
) -> Optional[AristonBaseDevice]:
    """Get ariston device"""
    api = await _async_connect(username, password)
    cloud_devices = await _async_discover(api)
    return _get_device(cloud_devices, api, gateway, is_metric, language_tag)



"""Evo device class for Ariston module."""
from __future__ import annotations

import logging
from datetime import datetime
from typing import Optional

from .const import (
    VelisPlantMode,
    EvoOneDeviceProperties,
    PlantData,
    WaterHeaterMode,
)
from .velis_base_device import AristonVelisBaseDevice

_LOGGER = logging.getLogger(__name__)


class AristonEvoOneDevice(AristonVelisBaseDevice):
    """Class representing a physical device, it's state and properties."""

    @property
    def plant_data(self) -> PlantData:
        """Final string to get plant data"""
        return PlantData.PD

    @property
    def consumption_type(self) -> str:
        """String to get consumption type"""
        return "Dhw"

    @property
    def water_heater_mode(self) -> type[WaterHeaterMode]:
        """Return the water heater mode class"""
        return VelisPlantMode

    @property
    def water_heater_current_temperature(self) -> Optional[float]:
        """Get water heater current temperature"""
        return self.data.get(EvoOneDeviceProperties.TEMP, None)

    @property
    def av_shw_value(self) -> Optional[int]:
        """Get average showers value"""
        return self.data.get(EvoOneDeviceProperties.AV_SHW, None)

    @property
    def water_heater_eco_value(self) -> Optional[int]:
        """Get water heater eco value"""
        return self.data.get(EvoOneDeviceProperties.ECO, None)

    @property
    def rm_tm_value(self) -> Optional[str]:
        """Get remaining time value"""
        return self.data.get(EvoOneDeviceProperties.RM_TM, None)

    @property
    def rm_tm_in_minutes(self) -> int:
        """Get remaining time value in minutes"""
        rm_tm = self.rm_tm_value
        if rm_tm is None:
            return -1
        time = datetime.strptime(rm_tm, "%H:%M:%S")
        return time.hour * 60 + time.minute

    def set_eco_mode(self, eco_mode: bool):
        """Set water heater eco_mode"""
        self.api.set_evo_eco_mode(self.gw, eco_mode)
        self.data[EvoOneDeviceProperties.ECO] = eco_mode

    async def async_set_eco_mode(self, eco_mode: bool):
        """Async set water heater eco_mode"""
        await self.api.async_set_evo_eco_mode(self.gw, eco_mode)
        self.data[EvoOneDeviceProperties.ECO] = eco_mode

    def set_water_heater_operation_mode(self, operation_mode: str):
        """Set water heater operation mode"""
        self.api.set_evo_mode(self.gw, VelisPlantMode[operation_mode])
        self.data[EvoOneDeviceProperties.MODE] = VelisPlantMode[operation_mode].value

    async def async_set_water_heater_operation_mode(self, operation_mode: str):
        """Async set water heater operation mode"""
        await self.api.async_set_evo_mode(self.gw, VelisPlantMode[operation_mode])
        self.data[EvoOneDeviceProperties.MODE] = VelisPlantMode[operation_mode].value

    @property
    def is_heating(self) -> Optional[bool]:
        """Get is the water heater heating"""
        return self.data.get(EvoOneDeviceProperties.STATE, None)

    @property
    def max_req_shower(self) -> Optional[str]:
        """Get maximum requestable shower"""
        return self.data.get(EvoOneDeviceProperties.MAX_REQ_SHW, None)

    @property
    def req_shower(self) -> Optional[str]:
        """Get requested shower"""
        return self.data.get(EvoOneDeviceProperties.REQ_SHW, None)

    def set_water_heater_number_of_showers(self, number_of_showers: int):
        """Set water heater number of showers"""
        self.api.set_evo_number_of_showers(self.gw, number_of_showers)
        self.data[EvoOneDeviceProperties.REQ_SHW] = number_of_showers

    async def async_set_water_heater_number_of_showers(self, number_of_showers: int):
        """Async set water heater number of showers"""
        await self.api.async_set_evo_number_of_showers(self.gw, number_of_showers)
        self.data[EvoOneDeviceProperties.REQ_SHW] = number_of_showers



"""Velis device class for Ariston module."""
from __future__ import annotations

import logging
from abc import ABC, abstractmethod
from typing import Any, Optional

from .ariston_api import AristonAPI
from .const import VelisDeviceProperties
from .velis_base_device import AristonVelisBaseDevice
from .device import AristonDevice

_LOGGER = logging.getLogger(__name__)


class AristonVelisDevice(AristonDevice, AristonVelisBaseDevice, ABC):
    """Class representing a physical device, it's state and properties."""

    def __init__(
        self,
        api: AristonAPI,
        attributes: dict[str, Any],
    ) -> None:
        super().__init__(api, attributes)
        self.plant_settings: dict[str, Any] = dict()

    @property
    @abstractmethod
    def anti_legionella_on_off(self) -> str:
        """Final string to get anti-legionella-on-off"""

    @property
    @abstractmethod
    def max_setpoint_temp(self) -> str:
        """Final string to get max setpoint temperature"""

    @property
    def water_anti_leg_value(self) -> Optional[bool]:
        """Get water heater anti-legionella value"""
        return self.plant_settings.get(self.anti_legionella_on_off, None)

    @property
    def proc_req_temp_value(self) -> Optional[float]:
        """Get process requested tempereature value"""
        return self.data.get(VelisDeviceProperties.PROC_REQ_TEMP, None)

    def set_antilegionella(self, anti_leg: bool):
        """Set water heater anti-legionella"""
        self.api.set_velis_plant_setting(
            self.plant_data,
            self.gw,
            self.anti_legionella_on_off,
            1.0 if anti_leg else 0.0,
            1.0 if self.plant_settings[self.anti_legionella_on_off] else 0.0,
        )
        self.plant_settings[self.anti_legionella_on_off] = anti_leg

    async def async_set_antilegionella(self, anti_leg: bool):
        """Async set water heater anti-legionella"""
        await self.api.async_set_velis_plant_setting(
            self.plant_data,
            self.gw,
            self.anti_legionella_on_off,
            1.0 if anti_leg else 0.0,
            1.0 if self.plant_settings[self.anti_legionella_on_off] else 0.0,
        )
        self.plant_settings[self.anti_legionella_on_off] = anti_leg

    def set_max_setpoint_temp(self, max_setpoint_temp: float):
        """Set water heater maximum setpoint temperature"""
        self.api.set_velis_plant_setting(
            self.plant_data,
            self.gw,
            self.max_setpoint_temp,
            max_setpoint_temp,
            self.plant_settings[self.max_setpoint_temp],
        )
        self.plant_settings[self.max_setpoint_temp] = max_setpoint_temp

    async def async_set_max_setpoint_temp(self, max_setpoint_temp: float):
        """Async set water heater maximum setpoint temperature"""
        await self.api.async_set_velis_plant_setting(
            self.plant_data,
            self.gw,
            self.max_setpoint_temp,
            max_setpoint_temp,
            self.plant_settings[self.max_setpoint_temp],
        )
        self.plant_settings[self.max_setpoint_temp] = max_setpoint_temp

    @property
    @abstractmethod
    def water_heater_maximum_setpoint_temperature_minimum(self) -> Optional[float]:
        """Get water heater maximum setpoint temperature minimum"""
        raise NotImplementedError

    @property
    @abstractmethod
    def water_heater_maximum_setpoint_temperature_maximum(self) -> Optional[float]:
        """Get water heater maximum setpoint maximum temperature"""
        raise NotImplementedError

    @property
    def water_heater_maximum_setpoint_temperature(self) -> Optional[float]:
        """Get water heater maximum setpoint temperature value"""
        return self.plant_settings.get(self.max_setpoint_temp, None)

    @property
    def water_heater_minimum_temperature(self) -> float:
        """Get water heater minimum temperature"""
        return 40.0

    @property
    def water_heater_maximum_temperature(self) -> Optional[float]:
        """Get water heater maximum temperature"""
        return self.water_heater_maximum_setpoint_temperature

    @property
    def water_heater_temperature_step(self) -> int:
        """Get water heater temperature step"""
        return 1

    @property
    def water_heater_temperature_decimals(self) -> int:
        """Get water heater temperature decimals"""
        return 0

    @property
    def water_heater_temperature_unit(self) -> str:
        """Get water heater temperature unit"""
        return "°C"



"""Lydos hybrid device class for Ariston module."""
from __future__ import annotations

import logging
from typing import Optional

from .const import (
    ConsumptionTimeInterval,
    ConsumptionType,
    LydosPlantMode,
    PlantData,
    SeDeviceSettings,
    LydosDeviceProperties,
)
from .evo_lydos_device import AristonEvoLydosDevice

_LOGGER = logging.getLogger(__name__)


class AristonLydosHybridDevice(AristonEvoLydosDevice):
    """Class representing a physical device, it's state and properties."""

    @property
    def plant_data(self) -> PlantData:
        """Final string to get plant data"""
        return PlantData.Se

    @property
    def anti_legionella_on_off(self) -> str:
        """Final string to get anti-legionella-on-off"""
        return SeDeviceSettings.SE_ANTILEGIONELLA_ON_OFF

    @property
    def consumption_type(self) -> str:
        """String to get consumption type"""
        return "DhwHeatingPumpElec%2CDhwResistorElec"

    @property
    def water_heater_mode(self) -> type[LydosPlantMode]:
        """Return the water heater mode class"""
        return LydosPlantMode

    @property
    def max_setpoint_temp(self) -> str:
        return SeDeviceSettings.SE_MAX_SETPOINT_TEMPERATURE

    @property
    def water_heater_maximum_setpoint_temperature_minimum(self) -> Optional[float]:
        """Get water heater maximum setpoint temperature minimum"""
        return self.plant_settings.get(
            SeDeviceSettings.SE_MAX_SETPOINT_TEMPERATURE_MIN, None
        )

    @property
    def water_heater_maximum_setpoint_temperature_maximum(self) -> Optional[float]:
        """Get water heater maximum setpoint maximum temperature"""
        return self.plant_settings.get(
            SeDeviceSettings.SE_MAX_SETPOINT_TEMPERATURE_MAX, None
        )

    @property
    def electric_consumption_for_water_last_two_hours(self) -> int:
        """Get electric consumption for water last value"""
        return self._get_consumption_sequence_last_value(
            ConsumptionType.DOMESTIC_HOT_WATER_HEATING_PUMP_ELECTRICITY,
            ConsumptionTimeInterval.LAST_DAY,
        )

    @property
    def permanent_boost_value(self) -> int:
        """Get permanent boost value"""
        return self.plant_settings.get(SeDeviceSettings.SE_PERMANENT_BOOST_ON_OFF, 0)

    @property
    def anti_cooling_value(self) -> int:
        """Get anti cooling value"""
        return self.plant_settings.get(SeDeviceSettings.SE_ANTI_COOLING_ON_OFF, 0)

    @property
    def anti_cooling_temperature_value(self) -> int:
        """Get anti cooling temperature value"""
        return self.plant_settings.get(SeDeviceSettings.SE_ANTI_COOLING_TEMPERATURE, 0)

    @property
    def anti_cooling_temperature_maximum(self) -> int:
        """Get anti cooling temperature maximum"""
        return self.plant_settings.get(SeDeviceSettings.SE_ANTI_COOLING_TEMPERATURE_MAX, 0)

    @property
    def anti_cooling_temperature_minimum(self) -> int:
        """Get anti cooling temperature minimum"""
        return self.plant_settings.get(SeDeviceSettings.SE_ANTI_COOLING_TEMPERATURE_MIN, 0)

    @property
    def night_mode_value(self) -> int:
        """Get night mode value"""
        return self.plant_settings.get(SeDeviceSettings.SE_NIGHT_MODE_ON_OFF, 0)

    @property
    def night_mode_begin_as_minutes_value(self) -> int:
        """Get night mode begin as minutes value"""
        return self.plant_settings.get(SeDeviceSettings.SE_NIGHT_BEGIN_AS_MINUTES, 0)

    @property
    def night_mode_begin_max_as_minutes_value(self) -> int:
        """Get night mode begin max as minutes value"""
        return self.plant_settings.get(SeDeviceSettings.SE_NIGHT_BEGIN_MAX_AS_MINUTES, 0)

    @property
    def night_mode_begin_min_as_minutes_value(self) -> int:
        """Get night mode begin min as minutes value"""
        return self.plant_settings.get(SeDeviceSettings.SE_NIGHT_BEGIN_MIN_AS_MINUTES, 0)

    @property
    def night_mode_end_as_minutes_value(self) -> int:
        """Get night mode end as minutes value"""
        return self.plant_settings.get(SeDeviceSettings.SE_NIGHT_END_AS_MINUTES, 0)

    @property
    def night_mode_end_max_as_minutes_value(self) -> int:
        """Get night mode end max as minutes value"""
        return self.plant_settings.get(SeDeviceSettings.SE_NIGHT_END_MAX_AS_MINUTES, 0)

    @property
    def night_mode_end_min_as_minutes_value(self) -> int:
        """Get night mode end min as minutes value"""
        return self.plant_settings.get(SeDeviceSettings.SE_NIGHT_END_MIN_AS_MINUTES, 0)

    def set_water_heater_operation_mode(self, operation_mode: str):
        """Set water heater operation mode"""
        self.api.set_lydos_mode(self.gw, LydosPlantMode[operation_mode])
        self.data[LydosDeviceProperties.MODE] = LydosPlantMode[operation_mode].value

    async def async_set_water_heater_operation_mode(self, operation_mode: str):
        """Async set water heater operation mode"""
        await self.api.async_set_lydos_mode(self.gw, LydosPlantMode[operation_mode])
        self.data[LydosDeviceProperties.MODE] = LydosPlantMode[operation_mode].value

    def set_water_heater_temperature(self, temperature: float):
        """Set water heater temperature"""
        self.api.set_lydos_temperature(self.gw, temperature)
        self.data[LydosDeviceProperties.REQ_TEMP] = temperature

    async def async_set_water_heater_temperature(self, temperature: float):
        """Async set water heater temperature"""
        await self.api.async_set_lydos_temperature(self.gw, temperature)
        self.data[LydosDeviceProperties.REQ_TEMP] = temperature

    def set_permanent_boost_value(self, boost: float) -> None:
        """Set permanent boost value"""
        self.api.set_velis_plant_setting(
            self.plant_data,
            self.gw,
            SeDeviceSettings.SE_PERMANENT_BOOST_ON_OFF,
            1.0 if boost else 0.0,
            1.0 if self.permanent_boost_value else 0.0,
        )
        self.plant_settings[SeDeviceSettings.SE_PERMANENT_BOOST_ON_OFF] = boost

    async def async_set_permanent_boost_value(self, boost: float) -> None:
        """Async set permanent boost value"""
        await self.api.async_set_velis_plant_setting(
            self.plant_data,
            self.gw,
            SeDeviceSettings.SE_PERMANENT_BOOST_ON_OFF,
            1.0 if boost else 0.0,
            1.0 if self.permanent_boost_value else 0.0,
        )
        self.plant_settings[SeDeviceSettings.SE_PERMANENT_BOOST_ON_OFF] = boost

    def set_anti_cooling_value(self, anti_cooling: float) -> None:
        """Set anti cooling value"""
        self.api.set_velis_plant_setting(
            self.plant_data,
            self.gw,
            SeDeviceSettings.SE_ANTI_COOLING_ON_OFF,
            1.0 if anti_cooling else 0.0,
            1.0 if self.anti_cooling_value else 0.0,
        )
        self.plant_settings[SeDeviceSettings.SE_ANTI_COOLING_ON_OFF] = anti_cooling

    async def async_set_anti_cooling_value(self, anti_cooling: float) -> None:
        """Async set anti cooling value"""
        await self.api.async_set_velis_plant_setting(
            self.plant_data,
            self.gw,
            SeDeviceSettings.SE_ANTI_COOLING_ON_OFF,
            1.0 if anti_cooling else 0.0,
            1.0 if self.anti_cooling_value else 0.0,
        )
        self.plant_settings[SeDeviceSettings.SE_ANTI_COOLING_ON_OFF] = anti_cooling

    def set_cooling_temperature_value(self, temperature: float) -> None:
        """Set cooling temperature value"""
        self.api.set_velis_plant_setting(
            self.plant_data,
            self.gw,
            SeDeviceSettings.SE_ANTI_COOLING_TEMPERATURE,
            temperature,
            self.anti_cooling_temperature_value,
        )
        self.plant_settings[SeDeviceSettings.SE_ANTI_COOLING_TEMPERATURE] = temperature

    async def async_set_cooling_temperature_value(self, temperature: float) -> None:
        """Async set cooling temperature value"""
        await self.api.async_set_velis_plant_setting(
            self.plant_data,
            self.gw,
            SeDeviceSettings.SE_ANTI_COOLING_TEMPERATURE,
            temperature,
            self.anti_cooling_temperature_value,
        )
        self.plant_settings[SeDeviceSettings.SE_ANTI_COOLING_TEMPERATURE] = temperature

    def set_night_mode_value(self, night_mode: float) -> None:
        """Set night mode value"""
        self.api.set_velis_plant_setting(
            self.plant_data,
            self.gw,
            SeDeviceSettings.SE_NIGHT_MODE_ON_OFF,
            1.0 if night_mode else 0.0,
            1.0 if self.night_mode_value else 0.0,
        )
        self.plant_settings[SeDeviceSettings.SE_NIGHT_MODE_ON_OFF] = night_mode

    async def async_set_night_mode_value(self, night_mode: float) -> None:
        """Async set night mode value"""
        await self.api.async_set_velis_plant_setting(
            self.plant_data,
            self.gw,
            SeDeviceSettings.SE_NIGHT_MODE_ON_OFF,
            1.0 if night_mode else 0.0,
            1.0 if self.night_mode_value else 0.0,
        )
        self.plant_settings[SeDeviceSettings.SE_NIGHT_MODE_ON_OFF] = night_mode

    def set_night_mode_begin_as_minutes_value(self, night_mode_begin_as_minutes: int) -> None:
        """Set night mode begin as minutes value"""
        self.api.set_velis_plant_setting(
            self.plant_data,
            self.gw,
            SeDeviceSettings.SE_NIGHT_BEGIN_AS_MINUTES,
            night_mode_begin_as_minutes,
            self.night_mode_begin_as_minutes_value,
        )
        self.plant_settings[SeDeviceSettings.SE_NIGHT_BEGIN_AS_MINUTES] = night_mode_begin_as_minutes

    async def async_set_night_mode_begin_as_minutes_value(self, night_mode_begin_as_minutes: int) -> None:
        """Async set night mode begin as minutes value"""
        await self.api.async_set_velis_plant_setting(
            self.plant_data,
            self.gw,
            SeDeviceSettings.SE_NIGHT_BEGIN_AS_MINUTES,
            night_mode_begin_as_minutes,
            self.night_mode_begin_as_minutes_value,
        )
        self.plant_settings[SeDeviceSettings.SE_NIGHT_BEGIN_AS_MINUTES] = night_mode_begin_as_minutes

    def set_night_mode_end_as_minutes_value(self, night_mode_end_as_minutes: int) -> None:
        """Set night mode end as minutes value"""
        self.api.set_velis_plant_setting(
            self.plant_data,
            self.gw,
            SeDeviceSettings.SE_NIGHT_END_AS_MINUTES,
            night_mode_end_as_minutes,
            self.night_mode_end_as_minutes_value,
        )
        self.plant_settings[SeDeviceSettings.SE_NIGHT_END_AS_MINUTES] = night_mode_end_as_minutes

    async def async_set_night_mode_end_as_minutes_value(self, night_mode_end_as_minutes: int) -> None:
        """Async set night mode end as minutes value"""
        await self.api.async_set_velis_plant_setting(
            self.plant_data,
            self.gw,
            SeDeviceSettings.SE_NIGHT_END_AS_MINUTES,
            night_mode_end_as_minutes,
            self.night_mode_end_as_minutes_value,
        )
        self.plant_settings[SeDeviceSettings.SE_NIGHT_END_AS_MINUTES] = night_mode_end_as_minutes



"""Lux2 device class for Ariston module."""
from __future__ import annotations

import logging

from .const import EvoDeviceProperties
from .evo_device import AristonEvoDevice

_LOGGER = logging.getLogger(__name__)

class AristonLux2Device(AristonEvoDevice):
    """Class representing a physical device, it's state and properties."""

    def set_water_heater_power_option(self, power_option: bool):
        """Set water heater power option"""
        self.api.set_lux_power_option(self.gw, power_option)
        self.data[EvoDeviceProperties.PWR_OPT] = power_option

    async def async_set_water_heater_power_option(self, power_option: bool):
        """Async set water heater power option"""
        await self.api.async_set_lux_power_option(self.gw, power_option)
        self.data[EvoDeviceProperties.PWR_OPT] = power_option



"""Nuos split device class for Ariston module."""
from __future__ import annotations

import logging
from typing import Optional

from .velis_device import AristonVelisDevice
from .const import (
    NuosSplitOperativeMode,
    NuosSplitProperties,
    PlantData,
    SlpDeviceSettings,
)

_LOGGER = logging.getLogger(__name__)


class AristonNuosSplitDevice(AristonVelisDevice):
    """Class representing a physical device, it's state and properties."""

    @property
    def plant_data(self) -> PlantData:
        """Final string to get plant data"""
        return PlantData.Slp

    @property
    def anti_legionella_on_off(self) -> str:
        """Final string to get anti-legionella-on-off"""
        return SlpDeviceSettings.SLP_ANTILEGIONELLA_ON_OFF

    @property
    def consumption_type(self) -> str:
        """String to get consumption type"""
        return "DhwHeatingPumpElec%2CDhwResistorElec"

    @property
    def water_heater_mode(self) -> type[NuosSplitOperativeMode]:
        """Return the water heater mode class"""
        return NuosSplitOperativeMode

    @property
    def max_setpoint_temp(self) -> str:
        return SlpDeviceSettings.SLP_MAX_SETPOINT_TEMPERATURE

    @property
    def water_heater_current_temperature(self) -> Optional[float]:
        """Get water heater current temperature"""
        return self.data.get(NuosSplitProperties.WATER_TEMP, None)

    @property
    def water_heater_target_temperature(self) -> Optional[float]:
        """Get water heater target temperature"""
        return self.data.get(NuosSplitProperties.COMFORT_TEMP, None)

    @property
    def water_heater_reduced_temperature(self) -> Optional[float]:
        """Get water heater reduced temperature"""
        return self.data.get(NuosSplitProperties.REDUCED_TEMP, None)

    @property
    def water_heater_maximum_setpoint_temperature_minimum(self) -> Optional[float]:
        """Get water heater maximum setpoint temperature minimum"""
        return self.plant_settings.get(
            SlpDeviceSettings.SLP_MAX_SETPOINT_TEMPERATURE_MIN, None
        )

    @property
    def water_heater_maximum_setpoint_temperature_maximum(self) -> Optional[float]:
        """Get water heater maximum setpoint maximum temperature"""
        return self.plant_settings.get(
            SlpDeviceSettings.SLP_MAX_SETPOINT_TEMPERATURE_MAX, None
        )

    @property
    def water_heater_minimum_setpoint_temperature(self) -> Optional[float]:
        """Get water heater minimum setpoint temperature value"""
        return self.plant_settings.get(
            SlpDeviceSettings.SLP_MIN_SETPOINT_TEMPERATURE, None
        )

    @property
    def water_heater_minimum_setpoint_temperature_minimum(self) -> Optional[float]:
        """Get water heater minimum setpoint temperature minimum"""
        return self.plant_settings.get(
            SlpDeviceSettings.SLP_MIN_SETPOINT_TEMPERATURE_MIN, None
        )

    @property
    def water_heater_minimum_setpoint_temperature_maximum(self) -> Optional[float]:
        """Get water heater minimum setpoint maximum temperature"""
        return self.plant_settings.get(
            SlpDeviceSettings.SLP_MIN_SETPOINT_TEMPERATURE_MAX, None
        )

    @property
    def water_heater_preheating_on_off(self) -> Optional[bool]:
        """Get water heater preheating on off"""
        return self.plant_settings.get(SlpDeviceSettings.SLP_PRE_HEATING_ON_OFF, None)

    @property
    def water_heater_heating_rate(self) -> Optional[float]:
        """Get water heater heating rate"""
        return self.plant_settings.get(SlpDeviceSettings.SLP_HEATING_RATE, None)

    @property
    def water_heater_boost(self) -> Optional[bool]:
        """Get water heater boost"""
        return self.data.get(NuosSplitProperties.BOOST_ON, None)

    @property
    def water_heater_mode_value(self) -> Optional[int]:
        """Get water heater mode value"""
        return self.data.get(NuosSplitProperties.OP_MODE, None)

    def set_water_heater_boost(self, boost: bool):
        """Set water heater boost"""
        self.api.set_nous_boost(self.gw, boost)
        self.data[NuosSplitProperties.BOOST_ON] = boost

    async def async_set_water_heater_boost(self, boost: bool):
        """Set water heater boost"""
        await self.api.async_set_nous_boost(self.gw, boost)
        self.data[NuosSplitProperties.BOOST_ON] = boost

    def _set_water_heater_temperature(self, temperature: float, reduced: float):
        """Set water heater temperature"""
        self.api.set_nuos_temperature(self.gw, temperature, reduced, self.water_heater_target_temperature, self.water_heater_reduced_temperature)
        self.data[NuosSplitProperties.PROC_REQ_TEMP] = temperature
        self.data[NuosSplitProperties.REDUCED_TEMP] = reduced

    def set_water_heater_temperature(self, temperature: float):
        """Set water heater temperature"""
        if len(self.data) == 0:
            self.update_state()
        reduced = self.water_heater_reduced_temperature
        if reduced is None:
            reduced = 0
        self._set_water_heater_temperature(temperature, reduced)

    async def _async_set_water_heater_temperature(
        self, temperature: float, reduced: float
    ):
        """Async set water heater temperature"""
        await self.api.async_set_nuos_temperature(self.gw, temperature, reduced, self.water_heater_target_temperature, self.water_heater_reduced_temperature)
        self.data[NuosSplitProperties.PROC_REQ_TEMP] = temperature
        self.data[NuosSplitProperties.REDUCED_TEMP] = reduced

    async def async_set_water_heater_temperature(self, temperature: float):
        """Async set water heater temperature"""
        if len(self.data) == 0:
            await self.async_update_state()
        reduced = self.water_heater_reduced_temperature
        if reduced is None:
            reduced = 0
        await self._async_set_water_heater_temperature(temperature, reduced)

    def set_water_heater_reduced_temperature(self, temperature: float):
        """Set water heater reduced temperature"""
        if len(self.data) == 0:
            self.update_state()
        current = self.water_heater_current_temperature
        if current is None:
            current = 0
        self._set_water_heater_temperature(current, temperature)

    async def async_set_water_heater_reduced_temperature(self, temperature: float):
        """Set water heater reduced temperature"""
        if len(self.data) == 0:
            await self.async_update_state()
        current = self.water_heater_current_temperature
        if current is None:
            current = self.water_heater_minimum_temperature
        await self._async_set_water_heater_temperature(current, temperature)

    def set_water_heater_operation_mode(self, operation_mode: str):
        """Set water heater operation mode"""
        self.api.set_nuos_mode(self.gw, NuosSplitOperativeMode[operation_mode])
        self.data[NuosSplitProperties.MODE] = NuosSplitOperativeMode[
            operation_mode
        ].value

    async def async_set_water_heater_operation_mode(self, operation_mode: str):
        """Async set water heater operation mode"""
        await self.api.async_set_nuos_mode(
            self.gw, NuosSplitOperativeMode[operation_mode]
        )
        self.data[NuosSplitProperties.MODE] = NuosSplitOperativeMode[
            operation_mode
        ].value

    def set_min_setpoint_temp(self, min_setpoint_temp: float):
        """Set water heater minimum setpoint temperature"""
        self.api.set_velis_plant_setting(
            self.plant_data,
            self.gw,
            SlpDeviceSettings.SLP_MIN_SETPOINT_TEMPERATURE,
            min_setpoint_temp,
            self.plant_settings[SlpDeviceSettings.SLP_MIN_SETPOINT_TEMPERATURE],
        )
        self.plant_settings[
            SlpDeviceSettings.SLP_MIN_SETPOINT_TEMPERATURE
        ] = min_setpoint_temp

    async def async_set_min_setpoint_temp(self, min_setpoint_temp: float):
        """Async set water heater minimum setpoint temperature"""
        await self.api.async_set_velis_plant_setting(
            self.plant_data,
            self.gw,
            SlpDeviceSettings.SLP_MIN_SETPOINT_TEMPERATURE,
            min_setpoint_temp,
            self.plant_settings[SlpDeviceSettings.SLP_MIN_SETPOINT_TEMPERATURE],
        )
        self.plant_settings[
            SlpDeviceSettings.SLP_MIN_SETPOINT_TEMPERATURE
        ] = min_setpoint_temp

    def set_preheating(self, preheating: bool):
        """Set water heater preheating"""
        self.api.set_velis_plant_setting(
            self.plant_data,
            self.gw,
            SlpDeviceSettings.SLP_PRE_HEATING_ON_OFF,
            preheating,
            self.plant_settings[SlpDeviceSettings.SLP_PRE_HEATING_ON_OFF],
        )
        self.plant_settings[SlpDeviceSettings.SLP_PRE_HEATING_ON_OFF] = preheating

    async def async_set_preheating(self, preheating: bool):
        """Async set water heater preheating"""
        await self.api.async_set_velis_plant_setting(
            self.plant_data,
            self.gw,
            SlpDeviceSettings.SLP_PRE_HEATING_ON_OFF,
            preheating,
            self.plant_settings[SlpDeviceSettings.SLP_PRE_HEATING_ON_OFF],
        )
        self.plant_settings[SlpDeviceSettings.SLP_PRE_HEATING_ON_OFF] = preheating

    def set_heating_rate(self, heating_rate: float):
        """Set water heater heating rate"""
        self.api.set_velis_plant_setting(
            self.plant_data,
            self.gw,
            SlpDeviceSettings.SLP_HEATING_RATE,
            heating_rate,
            self.plant_settings[SlpDeviceSettings.SLP_HEATING_RATE],
        )
        self.plant_settings[SlpDeviceSettings.SLP_HEATING_RATE] = heating_rate

    async def async_set_heating_rate(self, heating_rate: float):
        """Async set water heater heating rate"""
        await self.api.async_set_velis_plant_setting(
            self.plant_data,
            self.gw,
            SlpDeviceSettings.SLP_HEATING_RATE,
            heating_rate,
            self.plant_settings[SlpDeviceSettings.SLP_HEATING_RATE],
        )
        self.plant_settings[SlpDeviceSettings.SLP_HEATING_RATE] = heating_rate



"""Evo and lydos device class for Ariston module."""
from __future__ import annotations

import logging
from abc import ABC
from typing import Optional

from .velis_device import AristonVelisDevice
from .const import EvoLydosDeviceProperties

_LOGGER = logging.getLogger(__name__)


class AristonEvoLydosDevice(AristonVelisDevice, ABC):
    """Class representing a physical device, it's state and properties."""

    @property
    def water_heater_current_temperature(self) -> Optional[float]:
        """Get water heater current temperature"""
        return self.data.get(EvoLydosDeviceProperties.TEMP, None)

    @property
    def water_heater_target_temperature(self) -> Optional[float]:
        """Get water heater target temperature"""
        return self.data.get(EvoLydosDeviceProperties.REQ_TEMP, None)

    @property
    def av_shw_value(self) -> Optional[int]:
        """Get average showers value"""
        return self.data.get(EvoLydosDeviceProperties.AV_SHW, None)

    @property
    def is_heating(self) -> Optional[bool]:
        """Get is the water heater heating"""
        return self.data.get(EvoLydosDeviceProperties.HEAT_REQ, None)

    @property
    def is_antileg(self) -> Optional[bool]:
        """Is anti-legionella cycle running"""
        return self.data.get(EvoLydosDeviceProperties.ANTI_LEG, None)



"""Device class for Ariston module."""
from __future__ import annotations

import logging
from abc import ABC, abstractmethod
from typing import Optional

from .base_device import AristonBaseDevice

_LOGGER = logging.getLogger(__name__)


class AristonDevice(AristonBaseDevice, ABC):
    """Class representing a physical device, it's state and properties."""

    @property
    @abstractmethod
    def water_heater_temperature_decimals(self) -> int:
        """Abstract method for get water heater temperature decimals"""
        raise NotImplementedError

    @property
    @abstractmethod
    def water_heater_temperature_unit(self) -> str:
        """Abstract method for get water heater temperature unit"""
        raise NotImplementedError

    @property
    @abstractmethod
    def water_heater_temperature_step(self) -> int:
        """Abstract method for get water heater temperature step"""
        raise NotImplementedError

    @property
    @abstractmethod
    def water_heater_minimum_temperature(self) -> float:
        """Abstract method for get water heater minimum temperature"""
        raise NotImplementedError

    @property
    @abstractmethod
    def water_heater_target_temperature(self) -> Optional[float]:
        """Abstract method for get water heater target temperature"""
        raise NotImplementedError

    @property
    @abstractmethod
    def water_heater_maximum_temperature(self) -> Optional[float]:
        """Abstract method for get water heater maximum temperature"""
        raise NotImplementedError

    @abstractmethod
    def set_water_heater_temperature(self, temperature: float) -> None:
        """Abstract method for set water temperature"""
        raise NotImplementedError

    @abstractmethod
    async def async_set_water_heater_temperature(self, temperature: float) -> None:
        """Abstract method for async set water temperature"""
        raise NotImplementedError
